/*********************************************************************
 *
 *             Microchip USB C18 Firmware 
 *
 *********************************************************************
 * FileName:        winusb.h
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original HID mouse firmware (hid.h).
 * Jan Axelson           2/20/08    Renamed and modified for WinUSB.	
 ********************************************************************/
#ifndef WINUSB_H
#define WINUSB_H

/** I N C L U D E S **********************************************************/
#include "system\typedefs.h"

/** D E F I N I T I O N S ****************************************************/

/* Vendor-specific Requests */

#define WINUSB_REQUEST_1     0x01
#define WINUSB_REQUEST_2     0x02

/* WINUSB (vendor-specific) Interface Class Code */

#define WINUSB_INTF                    0xff

/******************************************************************************
 * Macro:           (bit) mWINUSBBulkRxIsBusy(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This macro is used to check if the bulk OUT endpoint is
 *                  busy (owned by SIE) or not.
 *                  Typical Usage: if(mWinUsbBulkRxIsBusy())
 *
 * Note:            None
 *****************************************************************************/

#define mWinUsbBulkRxIsBusy()              WINUSB_BD_BULK_OUT.Stat.UOWN

/******************************************************************************
 * Macro:           (bit) mWinUsbBulkTxIsBusy(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This macro is used to check if the bulk IN endpoint is
 *                  busy (owned by SIE) or not.
 *                  Typical Usage: if(mWinUsbBulkTxIsBusy())
 *
 * Note:            None
 *****************************************************************************/

#define mWinUsbBulkTxIsBusy()              WINUSB_BD_BULK_IN.Stat.UOWN

/******************************************************************************
 * Macro:           byte mWinUsbGetBulkRxLength(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          winusb_interrupt_rx_len
 *
 * Side Effects:    None
 *
 * Overview:        retrieves the number of bytes
 *                  copied to user's buffer by the most recent call to
 *                  WinUsbBulkRx function.
 *
 * Note:            None
 *****************************************************************************/

#define mWinUsbGetBulkRxLength()        winusb_bulk_rx_len

/******************************************************************************
 * Macro:           byte mWinUsbGetInterruptRxLength(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          winusb_interrupt_rx_len
 *
 * Side Effects:    None
 *
 * Overview:        retrieves the number of bytes
 *                  copied to user's buffer by the most recent call to
 *                  WinUsbInterruptRx function.
 *
 * Note:            None
 *****************************************************************************/

#define mWinUsbGetInterruptRxLength()        winusb_interrupt_rx_len

/******************************************************************************
 * Macro:           (bit) mWINUSBInterruptRxIsBusy(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This macro is used to check if the interrupt OUT endpoint is
 *                  busy (owned by SIE) or not.
 *                  Typical Usage: if(mWINUSBInterruptRxIsBusy())
 *
 * Note:            None
 *****************************************************************************/

#define mWinUsbInterruptRxIsBusy()              WINUSB_BD_INTERRUPT_OUT.Stat.UOWN

/******************************************************************************
 * Macro:           (bit) mWinUsbInterruptTxIsBusy(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This macro is used to check if the interrupt IN endpoint is
 *                  busy (owned by SIE) or not.
 *                  Typical Usage: if(mWinUsbInterruptTxIsBusy())
 *
 * Note:            None
 *****************************************************************************/

#define mWinUsbInterruptTxIsBusy()         WINUSB_BD_INTERRUPT_IN.Stat.UOWN

/** E X T E R N S ************************************************************/
extern byte winusb_interrupt_rx_len;

/** P U B L I C  P R O T O T Y P E S *****************************************/

void USBCheckWinUsbRequest(void);
void WinUsbInitEP(void);
byte WinUsbInterruptRx(char *buffer);
void WinUsbInterruptTx(char *buffer, byte len);
byte WinUsbBulkRx(char *buffer);
void WinUsbBulkTx(char *buffer, byte len);

#endif 
